% Shows that the implied trinomial tree recovers market prices of calls and
% puts.  Uses the implied Trinomial tree from Clewlow & Strickland's book
% "Implementing Derivatives Models, Chapter 5.
% By Fabrice Douglas Rouah.

clc; clear;

% Inline function for the Black Scholes call.
C = inline('s.*normcdf((log(s./K) + (r+v.^2./2).*T)./v./sqrt(T)) - K.*exp(-r.*T).*normcdf((log(s./K) + (r+v.^2./2).*T)./v./sqrt(T) - v.*sqrt(T))',...
	's','K','r','v','T');

% Inline function for the Black Scholes put.
P = inline('K.*exp(-r.*T).*normcdf(-(log(s./K) + (r+v.^2./2).*T)./v./sqrt(T) + v.*sqrt(T)) - s.*normcdf(-(log(s./K) + (r+v.^2./2).*T)./v./sqrt(T))',...
	's','K','r','v','T');

% Input option features.
T = 1;                  % Maturity.
Spot = 100;             % Spot price.
v = 0.15;               % Volatility skew.
r = 0.06;               % Risk free rate.
N = 4;                  % Number of steps.
maxSig = 0.2914;        % Maximum sigma.
dt = T/N;               % Time step.
dx = maxSig*sqrt(3*dt); % Stock price step.

% Create the strikes.
K = Spot*exp([N:-1:-N]'.*dx);

% Create the time steps.
time = [0:dt:T];

% Create the implied volatility surface, 
% and the calls and puts based on this surface.
% Note the functional form of the implied volatilty used
% by Clewlow and Strickland: IV(K,T) = a + b(K-Spot)^2*exp(-cT).
a = .2;
b = 0.000001;
c = 50;

for t=1:length(time);
	for k=1:length(K);
		% Implied vol surface
		IV(k,t) = a + b*(K(k) - Spot)^2*exp(-c*time(t));
		% Calls and puts
		if t>1
			Call(k,t) = C(Spot,K(k),r,IV(k,t),time(t));
			Put(k,t)  = P(Spot,K(k),r,IV(k,t),time(t));
		else
			Call(k,t) = max(Spot - K(k), 0);
			Put(k,t)  = max(K(k) - Spot, 0);
		end
	end
end
max(max(IV));

% Figure 5.3 and 5.6 of Clewlow and Strickland.
% Construct the Arrow-Debreu state prices.
% Use strike prices as the stock prices.
S = K;
Q = buildQ(S,N,Call,Put);

% Figure 5.6 of Clewlow & Strickland.
% Construct the trees for the lower (pd), middle (pm),
% and upper (pu) transition probabilities.
infl = exp(r*dt);          % Inflation factor.
[pu pm pd] = buildProbs(N,Q,infl,S);

% Maturity is time(n)
EuroAmer = 'E';

% Use the tree to price calls and puts and show that these prices
% are identical to the original prices used to construct the tree.
for n=N+1:-1:1
	for k=1:2*N+1
		Strike = S(k);
		% Resize the probability matrices for the correct maturity
		PU = pu(N-n+2:N+n, 1:n);
		PD = pd(N-n+2:N+n, 1:n);
		PM = pm(N-n+2:N+n, 1:n);
		% Resize the stock price vector for the correct maturity.
		SS = S(N-n+2:N+n);
		% Price the calls and puts
		RecoveredCall(k,n) = TreeOptions(SS,Strike,r,dt,PU,PM,PD,n-1,EuroAmer,'C');
		RecoveredPut(k,n)  = TreeOptions(SS,Strike,r,dt,PU,PM,PD,n-1,EuroAmer,'P');
	end
end

disp('---------- Original calls and recovered calls -------------')
Call
RecoveredCall

disp('---------- Original puts and recovered puts -------------')
Put
RecoveredPut

disp('---------- Price recovery error -------------')
Call_SSE = sum(sum((RecoveredCall - Call).^2))
Put_SSE  = sum(sum((RecoveredPut  - Put ).^2))

